#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define NBAND 3
#define FWHMPSF 2.5
#define THRESHOLD 5.
#define FILE_LENGTH 80
#define OPM_BIN "~/OPM/opm"
#define WCS_BIN "~/OPM/put_wcs_header"
#define LOGIN_CL "/home/nmatsuna/login.cl"

void usage(char *command);
void copy_login_cl(void);
double do_imstat(char *fits);
void get_xym(char *fits, double sigma, char *xym);
void do_opm(char *xym, char *catalog, char *match, char *param);
void put_wcs(char *fits, char *match, char *RA, char *DE);

int main(int argc, char *argv[]){
   int i;
   char *fits[NBAND], *catalog, *RA, *DE, *param;
   char xym[NBAND][FILE_LENGTH], match[NBAND][FILE_LENGTH];
   double sigma;
   if(argc!=8) usage(argv[0]);
   for(i=0;i<NBAND;i++){
      fits[i]=argv[i+1];
   }
   catalog=argv[4];
   RA=argv[5];
   DE=argv[6];
   param=argv[7];

   for(i=0;i<NBAND;i++){
      sigma=do_imstat(fits[i]);
      sprintf(xym[i],"%s.xym",fits[i]);
      sprintf(match[i],"%s.match",fits[i]);
      get_xym(fits[i],sigma,xym[i]);
      do_opm(xym[i],catalog,match[i],param);
      put_wcs(fits[i],match[i],RA,DE);
   }
   return 0;
}

void usage(char *command){
   fprintf(stderr,"usage -> %s [w1.fits] [w2.fits] [w3.fits] [catalog] [RA] [DE] [param]\n",command);
   exit(1);
}

void copy_login_cl(void){
   char *command;
   command=malloc((strlen(LOGIN_CL)+15)*sizeof(char));
   sprintf(command,"cp -rp %s .",LOGIN_CL);
   system(command);
   free(command);
}

double do_imstat(char *fits){
   char cl_file[]="cl_kadai2", log_file[]="log_kadai2", tmp_file[]="imstat.tmp";
   char command[FILE_LENGTH];
   double sigma;
   FILE *fp;
   if((fp=fopen(cl_file,"w"))==NULL){
      fprintf(stderr,"Failed to open %s\n",cl_file);
      exit(1);
   }
   fprintf(fp,"images\n");
   fprintf(fp,"imutil\n");
   fprintf(fp,"imstat %s fields=\"stddev\" format- nclip=2 lsigma=3. usigma=3. > %s\n",fits,tmp_file);
   fprintf(fp,"logout\n");
   fclose(fp);
   copy_login_cl();
   sprintf(command,"cl -old < %s > %s",cl_file,log_file);
   system(command);
   remove(cl_file);
   remove(log_file);
   if((fp=fopen(tmp_file,"r"))==NULL){
      fprintf(stderr,"Failed to open %s\n",tmp_file);
      goto error;
   }
   if(fscanf(fp,"%lf",&sigma)!=1) goto error;
   fclose(fp);
   remove(tmp_file);
   return sigma;
error:
   fprintf(stderr,"do_imstat failed for %s\n",fits);
   exit(1);
}

void get_xym(char *fits, double sigma, char *xym){
   char cl_file[]="cl_kadai2", log_file[]="log_kadai2";
   char command[FILE_LENGTH];
   FILE *fp;
   if((fp=fopen(cl_file,"w"))==NULL){
      fprintf(stderr,"Failed to open %s\n",cl_file);
      exit(1);
   }
   fprintf(fp,"digiphot\n");
   fprintf(fp,"apphot\n");
   fprintf(fp,"unlearn daofind\n");
   fprintf(fp,"unlearn datapars\n");
   fprintf(fp,"unlearn findpars\n");
   fprintf(fp,"findpars.threshold=%.1f\n",THRESHOLD);
   fprintf(fp,"datapars.fwhmpsf=%.1f\n",FWHMPSF);
   fprintf(fp,"datapars.sigma=%.1f\n",sigma);
   fprintf(fp,"daofind %s output=\"STDOUT\" verify- | txdump STDIN xcen,ycen,mag yes > %s\n",fits,xym);
   fprintf(fp,"logout\n");
   fclose(fp);
   copy_login_cl();
   sprintf(command,"cl -old < %s > %s",cl_file,log_file);
   system(command);
   remove(cl_file);
   remove(log_file);
}

void do_opm(char *xym, char *catalog, char *match, char *param){ 
   char *command;
   command=malloc((strlen(xym)+strlen(catalog)+strlen(match)+strlen(param))*sizeof(char));
   sprintf(command,"%s %s %s %s %s",OPM_BIN,xym,catalog,match,param);
   system(command);
   free(command);
}
     
void put_wcs(char *fits, char *match, char *RA, char *DE){
   char *command;
   command=malloc((strlen(fits)+strlen(match)+strlen(RA)+strlen(DE))*sizeof(char));
   sprintf(command,"%s %s %s %s %s",WCS_BIN,fits,match,RA,DE);
   system(command);
   free(command);
}

